/*
 * Copyright (c) 2000-2002 Silicon Graphics, Inc and LANL  All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it would be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 *
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 * Contact information:  Silicon Graphics, Inc., 1600 Amphitheatre Pkwy, 
 * Mountain View, CA  94043, or:
 * 
 * http://www.sgi.com 
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

#include <sys/types.h>
#include <sys/param.h>
#include <sys/sysmacros.h>
#include <sys/times.h>
#include <sys/utsname.h>
#include "csaacct.h"

#include <grp.h>
#include <malloc.h>
#include <math.h>
#include <memory.h>
#include <pwd.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#include "acctdef.h"
#include "acctmsg.h"
#include "csa_conf.h"
#include "sbu.h"

#include "csabuild.h"
#include "extern_build.h"

/*     ************************ NOTE 10/99 *******************************
 *	This code has been minimally changed so it compiles.  When
 *	an MPP record type is defined and used for CSA, this code
 *	needs to be modified for that new record type definition.
 *	This code is being pulled along as part of the port effort,
 *	but it's not currently used.
 *	*****************************************************************
 */

/*
 *	basic statistic macros for vectors
 *	- VMIN	: minimum value of a vector
 *	- VMAX	: maximum value of a vector
 *	- VAVG	: averagage (arithmetic mean) value of a vector
 *	- VMMA	: minimum, maximum and average values of a vector
 *	- VSDV	: standard deviation of a vector
 */
#define	VMIN(RES,CNT,VEC) {						\
	int	II;							\
									\
	RES = VEC[0];							\
	for (II=1; II<CNT; II++)					\
		RES = MIN(RES,VEC[II]);					\
}

#define	VMAX(RES,CNT,VEC) {						\
	int	II;							\
									\
	RES = VEC[0];							\
	for (II=1; II<CNT; II++)					\
		RES = MAX(RES,VEC[II]);					\
}

#define	VAVG(RES,CNT,VEC) {						\
	int	II;							\
									\
	RES = VEC[0];							\
	for (II=1; II<CNT; II++)					\
		RES += VEC[II];						\
	RES /= ((CNT) == 0 ? 1 : CNT);					\
}

#define	VMMA(Rmin,Rmax,Ravg,CNT,VEC) {					\
	int	II;							\
	double	TMP;							\
									\
	Rmin = Rmax = Ravg = VEC[0];					\
	for (II=1; II<CNT; II++) {					\
		Rmin = MIN(Rmin,VEC[II]);				\
		Rmax = MAX(Rmax,VEC[II]);				\
		Ravg += VEC[II];					\
	}								\
	Ravg /= ((CNT) == 0 ? 1 : CNT);					\
}

#define	VSDV(RES,AVG,CNT,VEC) {						\
	int	II;							\
	double	TMP;							\
									\
	RES = 0;							\
	for (II=0; II<CNT; II++) {					\
		TMP = VEC[II] - AVG;					\
		RES += TMP * TMP;					\
	}								\
	RES /= ((CNT) == 0 ? 1 : CNT);					\
	RES = sqrt(RES);						\
}

/*
 *	basic statistic macros for scattered vectors
 *	- SVMIN	: minimum value of a scattered vector
 *	- SVMAX	: maximum value of a scattered vector
 *	- SVAVG	: averagage (arithmetic mean) value of a scattered vector
 *	- SVMMA	: minimum, maximum and average values of a scattered vector
 *	- SVSDV	: standard deviation of a scattered vector
 */
#define	SVMIN(RES,CNT,VIND,VEC) {					\
	int	II;							\
									\
	RES = VEC[0];							\
	for (II=1; II<CNT; II++)					\
		RES = MIN(RES,VEC[VIND[II]]);				\
}

#define	SVMAX(RES,CNT,VIND,VEC) {					\
	int	II;							\
									\
	RES = VEC[0];							\
	for (II=1; II<CNT; II++)					\
		RES = MAX(RES,VEC[VIND[II]]);				\
}

#define	SVAVG(RES,CNT,VIND,VEC) {					\
	int	II;							\
									\
	RES = VEC[VIND[0]];						\
	for (II=1; II<CNT; II++)					\
		RES += VEC[VIND[II]];					\
	RES /= ((CNT) == 0 ? 1 : CNT);					\
}

#define	SVMMA(Rmin,Rmax,Ravg,CNT,VIND,VEC) {				\
	int	II;							\
	double	TMP;							\
									\
	Rmin = Rmax = Ravg = VEC[VIND[0]];				\
	for (II=1; II<CNT; II++) {					\
		Rmin = MIN(Rmin,VEC[VIND[II]]);				\
		Rmax = MAX(Rmax,VEC[VIND[II]]);				\
		Ravg += VEC[VIND[II]];					\
	}								\
	Ravg /= ((CNT) == 0 ? 1 : CNT);					\
}

#define	SVSDV(RES,AVG,CNT,VIND,VEC) {					\
	int	II;							\
	double	TMP;							\
									\
	RES = 0;							\
	for (II=0; II<CNT; II++) {					\
		TMP = VEC[VIND[II]] - AVG;				\
		RES += TMP * TMP;					\
	}								\
	RES /= ((CNT) == 0 ? 1 : CNT);					\
	RES = sqrt(RES);						\
}

/*
 *	define lower bounds for CPU times and Mbytes transferred used in
 *	sanity checks to avoid zero divides.
 */
#define	ZERO_CPU_TIME	0.00000005	/* half a micro second : this is ZERO */
#define	ZERO_IO		0.0000001	/* single byte : this is ZERO */

/*
 *	mpp expanded data is held in a single buffer. this macro assigns
 *	vectors to their offsets into this buffer as follows:
 */
#define	EXP_USER	0		/* first vector */
#define	EXP_RSYS	1		/* second vector */
#define	EXP_IO		2		/* third vector */
#define	NO_EXP_VECS	3		/* total number of vectors in buffer */

#define	SET_EXP_VECS(BUF,UTIME,RSTIME,IO,NPES) {			\
	UTIME = &BUF[EXP_USER * NPES];					\
	RSTIME = &BUF[EXP_RSYS * NPES];					\
	IO = &BUF[EXP_IO * NPES];					\
}

/*
 *	definitions for cluster analysis
 */
#define	CLUSTERS	5		/* target number of clusters */
#define	MIN_CLUSTERS	2		/* minimum required clusters */
#define	CLUSTER_STRETCH	4		/* cluster stretch factor */
#define	CLUSTER_TRIPS	20		/* number of iterations */
#define	METRIC(A,B)	ABS((A) - (B))	/* cluster metric */
#define	ZERO_METRIC	0.00001		/* cluster considered stable */

struct	cluster {
	int	active;			/* cluster is active */
	double	center;			/* center */
	int	index;			/* popolation count */
	int	*members;		/* population indices */
};

struct	cluster_def {
	int	no_clusters;		/* number of clusters */
	int	trip_count;		/* iterations */
	double	mean[NO_EXP_VECS];	/* averages */
	double	sd[NO_EXP_VECS];	/* std. deviations */
	double	*scaled;		/* scaled values */
	struct cluster	*cluster;	/* clusters */
};

static struct cluster_def cluster_def;

/*
 *	calculate number of active clusters
 */
#define	ACTIVE_CLUSTERS(NPES)	MAX(MIN(CLUSTERS, (NPES) / CLUSTER_STRETCH), \
					MIN_CLUSTERS)


/*
 *	I N T E R N A L    F U N C T I O N S
 *	====================================
 */
/*
 * cluster_init
 *
 *	initialize cluster structures
 *	allocate cluster member buffer
 *	set initial clusters centers to an even distribution over [-1,1]
 *	called once for every cluster analysis
 */
#undef  FUNC_NAME
#define FUNC_NAME	"cluster_init()"
static void
cluster_init(int npes)
{
	int	ind;

	for(ind = 0; ind < cluster_def.no_clusters; ind++) {
		if (cluster_def.cluster[ind].members != NULL) {
			free (cluster_def.cluster[ind].members);
		}
		cluster_def.cluster[ind].active    = 1;
		cluster_def.cluster[ind].index     = 0;

		/*
		 * calculate initial centers
		 */
		cluster_def.cluster[ind].center    = -1.0 + ind * 
					(2.0/( cluster_def.no_clusters-1.0));

		/*
		 * allocate cluster member buffer
		 */
		cluster_def.cluster[ind].members   = (int *)malloc(
							npes * sizeof(int));
		if (cluster_def.cluster[ind].members == NULL) {
			acct_err(ACCT_ABORT,
			       _("%s: There was insufficient memory available when allocating '%s'."),
				FUNC_NAME, "cluster members");
			return;
		}
	}

	return;
}


/*
 * cluster_def_init
 *
 *	initialize cluster definition structure
 *	called once for every command
 */
#undef  FUNC_NAME
#define FUNC_NAME	"cluster_def_init()"

static void
cluster_def_init(int npes, double *exp_mpp)
{
	int 	pe, no_clusters;
	double	*exp_utime, *exp_srtime, *exp_io;
	double	*sca_utime, *sca_srtime, *sca_io;

	/*
	 * allocate clusters 
	 */
	if (cluster_def.cluster != NULL) {
		free (cluster_def.cluster);
	}
	no_clusters = ACTIVE_CLUSTERS(npes);
	no_clusters *= CLUSTER_STRETCH;
	cluster_def.cluster = (struct cluster *)malloc(no_clusters * 
					sizeof(struct cluster));
	if (cluster_def.cluster == NULL) {
		acct_err(ACCT_ABORT,
		       _("%s: There was insufficient memory available when allocating '%s'."),
			FUNC_NAME, "clusters");
		return;
	}
	cluster_def.no_clusters = no_clusters;

	/*
	 * allocate scaled data buffer 
	 */
	if (cluster_def.scaled != NULL) {
		free (cluster_def.scaled);
	}
	cluster_def.scaled =
		(double *)malloc(NO_EXP_VECS * npes * sizeof(double));
	if (cluster_def.scaled == NULL) {
		acct_err(ACCT_ABORT,
		       _("%s: There was insufficient memory available when allocating '%s'."),
			FUNC_NAME, "scaled buffer");
		return;
	}

	SET_EXP_VECS(exp_mpp, exp_utime, exp_srtime, exp_io, npes);
	SET_EXP_VECS(cluster_def.scaled,
		sca_utime, sca_srtime, sca_io, npes);

	/*
	 * scale data
	 *
	 *	scale(x) = (x - x_avg) / x_sdev
	 */
	for (pe = 0; pe < npes; pe++) {
		sca_utime[pe] =
			cluster_def.sd[EXP_USER] > ZERO_CPU_TIME ?
			(exp_utime[pe] - cluster_def.mean[EXP_USER])
				/ cluster_def.sd[EXP_USER] : 0.0;
		sca_srtime[pe] = 
			cluster_def.sd[EXP_RSYS] > ZERO_CPU_TIME ?
			(exp_srtime[pe] - cluster_def.mean[EXP_RSYS])
				/ cluster_def.sd[EXP_RSYS] : 0.0;
		sca_io[pe] = 
			cluster_def.sd[EXP_IO] > ZERO_IO ?
			(exp_io[pe] - cluster_def.mean[EXP_IO])
				/ cluster_def.sd[EXP_IO] : 0.0;
	}

	return;
}


/*
 * assign_to_clusters
 *
 *	assign all values to a cluster
 */
#undef  FUNC_NAME
#define FUNC_NAME	"assign_to_clusters()"

static void
assign_to_clusters(int npes, double *data)
{
	int	pe, ind, cluster;
	double	mindist, distance[CLUSTERS * CLUSTER_STRETCH];

	/*
	 * re-populate clusters
	 */
	for (ind = 0; ind < cluster_def.no_clusters; ind++) {
		cluster_def.cluster[ind].index = 0;

	}

	for (pe = 0; pe < npes; pe++) {

		/*
		 * calc the distance to the cluster centers
		 */
		for(ind = 0; ind < cluster_def.no_clusters; ind++) {
			if (cluster_def.cluster[ind].active == 0) {
				distance[ind] = -1;	/* not valid */
			} else {
				distance[ind] = METRIC(data[pe],
					cluster_def.cluster[ind].center);
			}
		}

		/*
		 * find the nearest cluster
		 */
		mindist = -1;				/* not valid */
		for(cluster = 0, ind = 0; ind < cluster_def.no_clusters; ind++){
			if (cluster_def.cluster[ind].active == 0) {
				continue;
			}
			if ((mindist < 0) || (distance[ind] < mindist)) {
				mindist = distance[ind];
				cluster = ind;
			}
		}

		/*
		 * add it to the nearest cluster
		 */
		ind = cluster_def.cluster[cluster].index;
		cluster_def.cluster[cluster].members[ind] = pe;
		cluster_def.cluster[cluster].index++;
	}

	return;
}


/*
 * cluster_change
 *
 *	- recalculate cluster centers, return number of changes
 *
 */
#undef  FUNC_NAME
#define FUNC_NAME	"cluster_change()"

static int
cluster_change(double *data)
{
	int	ind, moved;
	double	old_center;

	/*
	 * re-evaluate cluster centers
	 */
	for (ind = 0, moved = 0; ind < cluster_def.no_clusters; ind++) {
		if (cluster_def.cluster[ind].active == 0) {
			continue;
		}
		if (cluster_def.cluster[ind].index > 0) {
			old_center = cluster_def.cluster[ind].center;
			SVAVG(cluster_def.cluster[ind].center,
				cluster_def.cluster[ind].index,
				cluster_def.cluster[ind].members, data);
			if (METRIC(old_center,cluster_def.cluster[ind].center) >
						ZERO_METRIC) {
				moved++;
			}
		} else {
			cluster_def.cluster[ind].active = 0;
		}
	}

	return moved;
}


/*
 * cluster_join
 *
 *	join adjacent clusters
 */
static void
cluster_join()
{
	struct cluster	*cur, *join;
	int		cur_cl, join_cl, join_index;

	cur_cl = 0;

	/*
	 * loop through all clusters
	 */
	while (cur_cl < cluster_def.no_clusters) {

		/*
		 * find an active cluster
		 */
		if (cluster_def.cluster[cur_cl].active == 1) {
			join_cl = cur_cl + 1;
			cur = &cluster_def.cluster[cur_cl];

			/*
			 * join all adjacent active clusters to current cluster
			 */
			while (join_cl <  cluster_def.no_clusters) {
				if (cluster_def.cluster[join_cl].active == 0) {
					break;
				}
				join = &cluster_def.cluster[join_cl];

				/*
				 * join all the members to the current cluster
				 */
				for (join_index = 0;
						join_index < join->index;
						join_index++,
						cur->index++) {
					cur->members[cur->index] =
						join->members[join_index];
				}
				join->active = 0;
				join_cl++;
			}
		}
		cur_cl++;
	}

	return;
}


/*
 * cluster_print
 *
 *	print cluster stats
 */
#undef  FUNC_NAME
#define FUNC_NAME	"cluster_print()"

static void
cluster_print(int npes, double *raw_data, char *ident)
{
	int	ind, any, active;
	double	min, max, avg, sd;

	/*
	 * count number of active clusters
	 */
	for (ind = 0, active = 0; ind < cluster_def.no_clusters; ind++) {
		if (cluster_def.cluster[ind].active) {
			active++;
		}
	}

	/*
	 * check if 
	 * - less then one cluster is active or
	 * - more than ACTIVE_CLUSTERS are active or
	 * - the trip count was exceeded
	 */
	if ((active <= 1) ||
	    (active > ACTIVE_CLUSTERS(npes)) ||
	    (cluster_def.trip_count < 0)) {

		/*
		 * things didn't turn out as expected
		 */
		printf("# MPP Clusters    ");
		printf("%10.10s %10.10s %10.10s %10.10s   %s ",
			"     NA   ", "     NA   ",
			"     NA   ", "     NA   ", ident);
		printf(_("(*** No Cluster could be identified ***)\n"));
		any = 1;
	} else {

		/*
		 * show the clusters
		 */
		for (ind = 0, any = 0; ind < cluster_def.no_clusters; ind++) {
			if (cluster_def.cluster[ind].active) {
				SVMMA(min, max, avg,
					cluster_def.cluster[ind].index,
					cluster_def.cluster[ind].members,
					raw_data);
				SVSDV(sd, avg, cluster_def.cluster[ind].index, 
					cluster_def.cluster[ind].members,
					raw_data);
				if (any == 0) {
					printf("# MPP Clusters    ");
					any = 1;
				} else {
					printf("# MPP             ");
				}
				printf("%10.2f %10.2f %10.2f %10.2f   %s ",
					min, max, avg, sd, ident);
				printf(_("(No. of PEs in Cluster : %d)\n"),
					cluster_def.cluster[ind].index);
			}
		}
	}

	if (any) {
		printf("# MPP             \n");
	}

	return;
}


/*
 * cluster_analysis
 *
 *	One dimensional cluster analysis done as follows:
 *	
 *	1) Input data is scaled by:
 *	
 *		scale(x) = (x - x_avg) / x_sdv
 *	
 *		with	x_avg = arithmetic mean value
 *		and	x_sdv = standard deviation
 *	
 *	2) Initial clusters are generated with centers evenly distributed
 *	   over the interval [-1,1]
 *	
 *	3) All data elements are assigned to their nearest cluster. Since
 *	   this is a simple one-dimensional analysis the metric used is
 *	   simply 
 *	
 *		metric(x) = |x - x_avg|
 *	
 *	4) The cluster centers for all clusters are re-caclulated as the
 *	   arithmetic mean of the scaled values of all their members.
 *	   If no cluster center has moved, join adjacent clusters.
 *	
 *	5) Steps 3) and 4) are done again until either 
 *	   - no cluster center was moved
 *	   - an iteration count was exceeded.
 *	   The first case is considered success, the later failure.
 *
 *	6) If the cluster analysis was successful, adjacent clusters
 *	   are joined. 
 *
 *	7) Finaly, all non-empty clusters
 *	   are printed (statistics from the non-scaled data).
 *	
 */
#undef  FUNC_NAME
#define FUNC_NAME	"cluster_analysis()"

static void
cluster_analysis(int npes, double *exp_mpp, int index, char *ident)
{
	double	*raw_data, *data;
	int	pe;

	/*
	 * initialize clusters and data vectors
	 */
	cluster_init(npes);
	raw_data = &exp_mpp[npes * index];
	data = &cluster_def.scaled[npes * index];

	/*
	 * loop until clusters are stable or trip count exhausted
	 */
	cluster_def.trip_count = CLUSTER_TRIPS;

	while (cluster_def.trip_count-- >= 0) {

		/*
		 * re-assign all data values to the nearest cluster
		 */
		assign_to_clusters(npes, data);

		/*
		 * break if clusters didn't change
		 */
		if (cluster_change(data) == 0) {
			break;
		}
	};

	/*
	 * join adjacent clusters and show the results
	 */
	cluster_join();
	cluster_print(npes, raw_data, ident);

	return;
}


/*
 * cluster_stats
 */
#undef  FUNC_NAME
#define FUNC_NAME	"cluster_stats()"

static void
cluster_stats(struct acctmpp *mpp, double *exp_mpp)
{
	int	pe, npes;
	double	*exp_utime, *exp_srtime, *exp_io;

	if (exp_mpp == NULL) {
		return;
	}
	npes = mpp->ac_mppe;
	SET_EXP_VECS(exp_mpp, exp_utime, exp_srtime, exp_io, npes);

	cluster_def_init(npes, exp_mpp);
	cluster_analysis(npes, exp_mpp, EXP_USER,
			"Seconds User CPU Time");
	cluster_analysis(npes, exp_mpp, EXP_RSYS,
			"Seconds System CPU Time");
	cluster_analysis(npes, exp_mpp, EXP_IO,
			"Mbytes transferred");

	return;
}


/*
 *	E X P O R T E D    F U N C T I O N S
 *	====================================
 */
/*
 * mpp_expand
 *
 *	allocate and return buffer of the expanded and scaled mpp data
 */
#undef	FUNC_NAME
#define	FUNC_NAME	"mpp_expand()"

double *
mpp_expand(struct acctmpp *mpp)
{
	double		*exp_mpp;
	double		*exp_utime, *exp_srtime, *exp_io;
	struct acctpe	*mppe;
	int		pe, npes;

	/*
	 * allocate expanded data buffer
	 */
	npes = mpp->ac_mppe;
	exp_mpp = (double *)malloc(NO_EXP_VECS * npes * sizeof(double));
	if (exp_mpp == NULL) {
		acct_err(ACCT_ABORT,
		       _("%s: There was insufficient memory available when allocating '%s'."),
			FUNC_NAME, "expansion buffer");
		return NULL;
	}

	/*
	 * expand and scale data
	 *	times are in seconds
	 *	IO is in Mbytes
	 */
	SET_EXP_VECS(exp_mpp, exp_utime, exp_srtime, exp_io, npes);
	mppe = mpp->ac_mpp;
	for (pe = 0; pe < npes; pe++) {
		exp_utime[pe]  = mppe[pe].utime;
		exp_srtime[pe] = mppe[pe].srtime;
		exp_io[pe]     = mppe[pe].io;
		exp_utime[pe]  = TIME_2_SECS(exp_utime[pe]);
		exp_srtime[pe] = TIME_2_SECS(exp_srtime[pe]);
		exp_io[pe]   = (double)(exp_io[pe]) / (8 * 1024 * 1024);
	}

	return exp_mpp;
}


/*
 * mpp_percentage
 *
 *	allocate and calculate percentages
 */
#undef  FUNC_NAME
#define FUNC_NAME	"mpp_percentage()"

double *
mpp_percentage(struct acctent *acctrec, double *exp_mpp)
{
	int	pe, npes;
	double	*exp_utime, *exp_srtime, *exp_io;
	double	*exp_perc, *exp_utime_perc, *exp_srtime_perc, *exp_io_perc;
	double	time_100, ctime;
	double	io_100 = 0.0;

	if (exp_mpp == NULL) {
		return NULL;
	}
	/*
	 * allocate percentage buffer
	 */
	npes = acctrec->mpp->ac_mppe;
	exp_perc = (double *)malloc(NO_EXP_VECS * npes * sizeof(double));
	if (exp_perc == NULL) {
		acct_err(ACCT_ABORT,
		       _("%s: There was insufficient memory available when allocating '%s'."),
			FUNC_NAME,"percentage buffer");
		return NULL;
	}
	SET_EXP_VECS(exp_mpp, exp_utime, exp_srtime, exp_io, npes);
	SET_EXP_VECS(exp_perc, exp_utime_perc,
			exp_srtime_perc, exp_io_perc, npes);

	time_100 = TIME_2_SECS(acctrec->csa->ac_etime);
	time_100 = time_100 > ZERO_CPU_TIME ? (100.0 / time_100) : 0.0;
	if (acctrec->io) {
		io_100 = (double)(acctrec->io->ac_chr + 
			acctrec->io->ac_chw) / ( 8 * 1024 * 1024);
		io_100 = io_100 > ZERO_IO ? (100.0 / io_100) : 0.0;
	}

	for (pe = 0; pe < npes; pe++) {
		exp_utime_perc[pe]  = exp_utime[pe] * time_100;
		exp_srtime_perc[pe] = exp_srtime[pe] * time_100;
		exp_io_perc[pe]   = exp_io[pe] * io_100;
		
	}

	return exp_perc;
}


/*
 * mpp_basic_stats
 *
 *	print basic mpp statistics
 */
#undef  FUNC_NAME
#define FUNC_NAME	"mpp_basic_stats()"

void
mpp_basic_stats(struct acctmpp *mpp)
{
	printf("# MPP             \n");
	printf("# MPP Usage       ");
	printf("PE's= %d", mpp->ac_mppe);
	printf(", BESU's= %d", mpp->ac_mpbesu);
	printf(", PE Memory High Water= %4.2f MBytes",
		KB_2_MB(mpp->ac_himem));
	printf("\n");
	printf("# MPP             \n");

	return;
}


/*
 * mpp_extended_stats
 *
 *	calculate and print extended mpp statistics
 *		average, minimum, maximum and std. deviation
 *
 *	initiate cluster analysis
 */
#undef  FUNC_NAME
#define FUNC_NAME	"mpp_extended_stats()"
void
mpp_extended_stats(struct acctmpp *mpp, double *exp_mpp)
{
	double	*exp_utime, *exp_srtime, *exp_io;
	double	mp_umin, mp_umax, mp_uavg, mp_usd;
	double	mp_smin, mp_smax, mp_savg, mp_ssd;
	double	mp_imin, mp_imax, mp_iavg, mp_isd;
	double	mp_user, mp_system, mp_mem;
	int	pe, npes;

	if (exp_mpp == NULL) {
		return;
	}

	/*
	 * initialize
	 */
	npes = mpp->ac_mppe;
	SET_EXP_VECS(exp_mpp, exp_utime, exp_srtime, exp_io, npes);

	/*
	 * get min, max and avg
	 */
	VMMA(mp_umin, mp_umax, mp_uavg, npes, exp_utime);
	VMMA(mp_smin, mp_smax, mp_savg, npes, exp_srtime);
	VMMA(mp_imin, mp_imax, mp_iavg, npes, exp_io);

	/*
	 * get standard deviation
	 */
	VSDV(mp_usd, mp_uavg, npes, exp_utime);
	VSDV(mp_ssd, mp_savg, npes, exp_srtime);
	VSDV(mp_isd, mp_iavg, npes, exp_io);

	/*
	 * and print it
	 */
	printf("# MPP Statistic   "
	      " Minimum    Maximum    Average   Std. Dev. \n");
	printf("# MPP             "
	      "========== ========== ========== ==========\n");
	printf("# MPP             "
		"%10.2f %10.2f %10.2f %10.2f "
		"  Seconds User CPU Time\n",
		mp_umin, mp_umax, mp_uavg, mp_usd);
	printf("# MPP             "
		"%10.2f %10.2f %10.2f %10.2f "
		"  Seconds System CPU Time\n",
		mp_smin, mp_smax, mp_savg, mp_ssd);
	printf("# MPP             "
		"%10.2f %10.2f %10.2f %10.2f "
		"  Mbytes transferred\n",
		mp_imin, mp_imax, mp_iavg, mp_isd);
	printf("# MPP             \n");

	/*
	 * do the cluster analysis
	 */
	cluster_def.mean[EXP_USER] = mp_uavg;
	cluster_def.mean[EXP_RSYS] = mp_savg;
	cluster_def.mean[EXP_IO] = mp_iavg;
	cluster_def.sd[EXP_USER] = mp_usd;
	cluster_def.sd[EXP_RSYS] = mp_ssd;
	cluster_def.sd[EXP_IO] = mp_isd;
	cluster_stats(mpp, exp_mpp);

	return;
}


/*
 * mpp_xgraph
 *
 *	xgraph style output of mpp data
 */
#undef  FUNC_NAME
#define FUNC_NAME	"mpp_xgraph()"

void
mpp_xgraph(struct acctent *acctrec, double *exp_perc)
{
	int	pe, npes;
	double	*exp_utime_perc, *exp_srtime_perc, *exp_io_perc;

	if (exp_perc == NULL) {
		return;
	}
	npes = acctrec->mpp->ac_mppe;
	SET_EXP_VECS(exp_perc, exp_utime_perc,
			exp_srtime_perc, exp_io_perc, npes);

	printf("TitleText: MPP Utilization Breakdown (%s)\n",
			acctrec->csa->ac_comm);
	printf("XUnitText: PE#\n");
	printf("YUnitText: %% of Wallclock Time or Total IO\n");
	printf("YLowLimit: 0\n");
	printf("YHighLimit: 100\n");

	printf("\n\"User CPU\"\n");
	for (pe = 0; pe < npes; pe++) {
		printf("%d %10.2f\n",pe,exp_utime_perc[pe]);
	}

	printf("\n\"System CPU\"\n");
	for (pe = 0; pe < npes; pe++) {
		printf("%d %10.2f\n",pe,exp_srtime_perc[pe]);
	}

	printf ("\n\"Relative IO\"\n");
	for (pe = 0; pe < npes; pe++) {
		printf("%d %10.2f\n",pe,exp_io_perc[pe]);
	}

	return;
}


/*
 * mpp_dump
 *
 *	dump mpp data
 */
#undef  FUNC_NAME
#define FUNC_NAME	"mpp_dump()"

void
mpp_dump(struct acctmpp *mpp, double *exp_mpp)
{
	double	*exp_utime, *exp_srtime, *exp_io;
	int 	npes, pe;

	if (exp_mpp == NULL) {
		return;
	}
	npes = mpp->ac_mppe;
	SET_EXP_VECS(exp_mpp, exp_utime, exp_srtime, exp_io, npes);
	printf("# MPP Dump   "
		"PE#   User CPU   Sys CPU    Mbytes   \n");
	printf("# MPP        "
		"       Seconds   Seconds    Xferred  \n");
	printf("# MPP        "
		"==== ========== ========== ==========\n");

	/*
	 * loop through entire PE information and dump it
	 */
	for(pe = 0; pe < mpp->ac_mppe; pe++) {
		printf("# MPP        %4d %10.2f %10.2f %10.2f\n",
			pe, exp_utime[pe], exp_srtime[pe], exp_io[pe]);
	}
	printf("# MPP             \n");

	return;
}
