/*
 * Copyright (c) 2000-2002 Silicon Graphics, Inc and LANL  All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it would be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 *
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 * Contact information:  Silicon Graphics, Inc., 1600 Amphitheatre Pkwy, 
 * Mountain View, CA  94043, or:
 * 
 * http://www.sgi.com 
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

#include <sys/types.h>
#include <sys/param.h>
#include <sys/sysmacros.h>
#include <sys/times.h>
#include <sys/utsname.h>
#include "csaacct.h"

#include <grp.h>
#include <malloc.h>
#include <memory.h>
#include <pwd.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#include "acctdef.h"
#include "acctmsg.h"
#include "csa_conf.h"
#include "sbu.h"

#include "csabuild.h"
#include "extern_build.h"


/*
 *	process_tape() - process a Tape daemon record.
 */
int
process_tape(int fd, int findex, int mark)
{
	static	int	tso_flag = 0;
	static	time_t  lasttime = 0;
	struct	tpja	*tb;
	struct	tphdr	*thdr;
	time_t	btime;
	uint64_t jobid;		/* Job IDentifier */
	struct	segment *sp;	/* active segment pointer */
	struct Frec	*Fptr;	/* new File record pointer */

	/*
	 * On IRIX systems, the records in the tape file are ordered by
	 * the btime field.  btime is not necessarily before EOJ time,
	 * but actime is.  Thus, use btime to make sure that the records
	 * are in the correct order and to figure out which up period the
	 * record is in.  Use actime when determining which segment the
	 * record should go in.
	 */
	btime = acctent.tpbs->btime;
	if (lasttime != 0) {
		if (lasttime > btime) {
			/*
			 * If time stamps are out of order, keep on going.
			 * Assume that the records belong in the current
			 * or later up periods.
			 */
			if (!tso_flag) {
				tso_flag++;
				acct_err(ACCT_WARN,
				       _("The %s records were out of time order in file '%s%d' near offset %lld due to a date change or an invalid file."),
					"tape",
					pacct, findex, Pacct_offset);
			}
		}

	} else {
		if (btime < uptime[0].up_start) {
		    if (btime < EARLY_FUDGE) {
			char btime_str[26];
			time_t	time = EARLY_FUDGE;

			strcpy(btime_str, ctime(&btime));
			Nerror("REC_TAPE: EARLY_FUDGE error (%d - %24.24s) ",
				btime, btime_str);
			Nerror("< (%d - %24.24s).\n", time, ctime(&time));
			acct_err(ACCT_WARN,
				_("An earlier than expected time stamp was discovered in the file '%s%d' near offset (%lld).  Examine the data set."),
				pacct, findex, Pacct_offset);

			if (i_opt) {
				acct_err(ACCT_INFO, 
					_("The record was ignored."));
				if (db_flag > 2) {
				 	Dump_tpbs_rec(acctent.tpbs);
				}
				return(RC_BAD);
			}
		    }
		    if (upind != 0) {
				if (db_flag > 2) {
				 	Dump_tpbs_rec(acctent.tpbs);
				 }
				 acct_err(ACCT_ABORT,
				        _("An earlier than expected time stamp was discovered in the file '%s%d' near offset (%lld).  Examine the data set."),
					pacct, findex, Pacct_offset);

		    } else {
				Shift_uptime(REC_TAPE, pacct, findex);
		    }
	    }
	}
	lasttime = acctent.tpbs->btime;
	while(uptime[upind].up_stop < btime) {
		if (db_flag > 3) {
			Ndebug("process_tpbs(4): Past stop time(%d) going "
				"to next tape uptime(%d) - %s%d.\n",
				uptime[upind].up_stop, upind+1, pacct, findex);
		}
		upind++;
		if (upind > upIndex) {
			return(RC_UPIND);
		}
	}
	btime = acctent.tpbs->actime;
	jobid = acctent.tpbs->jid;
	if ((sp = get_segptr(REC_TAPE, upind, jobid, btime, 0L, 0L)) ==
			NOJOBFOUND) {
		return(RC_NOJOB);
	}

	/*
	 *	Insert File record at end of the Tape chain.
	 */
	Total_frecs++;
	Fptr = get_frec(Pacct_offset, mark, REC_TAPE);
	*(sp->sg_ltape) = Fptr;
	sp->sg_ltape = &(Fptr->fp_next);

	return(RC_GOOD);
}
