/*
 * Copyright (c) 2000-2002 Silicon Graphics, Inc and LANL  All Rights Reserved.
 * Copyright (c) 2004 Silicon Graphics, Inc All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it would be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 *
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 * Contact information:  Silicon Graphics, Inc., 1500 Crittenden Lane,
 * Mountain View, CA  94043, or:
 * 
 * http://www.sgi.com 
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

#include <sys/types.h>
#include <sys/param.h>
#include <sys/sysmacros.h>
#include <sys/times.h>
#include <sys/utsname.h>

#include <grp.h>
#include <malloc.h>
#include <memory.h>
#include <pwd.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#include "acctdef.h"
#include "acctmsg.h"
#include "csa_conf.h"
#include "sbu.h"

#include "csaacct.h"
#include "csaja.h"
#include "extern_ja.h"

#define TIMEBUF_MAX	40

/*
 *	bld_nqs() - Add an nqs record to the nqs list
 */
void
bld_nqs()
{
	struct	nqsbs	*nqs;
	struct	nqsja	*nq;
	char	*uname;

	/*
	 *	Make sure the user wants a daemon report information.
	 */
	if (!d_opt) {
		return;
	}

	/*
	 *	Reject this entry if the user is being selective and does
	 *	not want to see this one.
	 */
	nqs = acctrec.nbs;
	if ((j_opt && (nqs->jid != s_jid)) || (a_opt && (nqs->ash != s_ash)) ||
	    (u_opt && (nqs->uid != s_uid)) || (p_opt && (nqs->prid != s_prid))){
		if (db_flag > 3) {
		    fprintf(stderr,
		      "%s: NQS record deselected - uid(%d, %d), jid(0x%llx, 0x%llx), ash(%lld, %lld), prid(%lld, %lld).\n",
			Prg_name,
			nqs->uid, s_uid, nqs->jid, s_jid,
			nqs->ash, s_ash, nqs->prid, s_prid);
		}
		return;
	}

	/*
	 *	Output the NQS record information.
	 */
	uname = uid_to_name(nqs->uid);
	if (!strcmp(uname, "?") ) {
		uname = "Unknown";
	}
	fprintf(stdout,
	      "# NQS    (%#18llx) NQS Record (%4d) %-8s %-16s %-16s %-16s %8lld",
		nqs->jid, nqs->type, uname, nqs->quename, nqs->reqname,
		nqs->machname, nqs->seqno);
	if (l_opt) {
		time_t	timb;
		timb = nqs->time;
		fprintf(stdout, " %8.4f %8.4f %8lld %5d %#18llx %#18llx"
			" %20.20s\n",
			TIME_2_SECS(nqs->utime), TIME_2_SECS(nqs->stime),
			nqs->qwtime, nqs->qtype, nqs->prid, nqs->ash,
			ctime(&timb)+4);
	} else {
		fprintf(stdout, "\n");
	}

	/*
	 *	We're only expecting to see NQ_INIT records after a
	 *	restart, ignore anything else that might have snuck in.
	 */
	if (nqs->type != NQ_INIT) {
		return;
	}

	if ((nq = (struct nqsja *)malloc(sizeof(struct nqsja))) == NULL) {
		acct_perr(ACCT_ABORT, errno,
			_("There was insufficient memory available when allocating '%s'."),
			"nqs entry");
	}

	if (nqhdr->head == NULL) {
		nqhdr->head = nqhdr->last = nq;

	} else {
		nqhdr->last->nxt = nq;
		nqhdr->last = nq;
	}

	(void)strncpy(nq->reqname, nqs->reqname, 16);
	nq->type = nqs->type;
	nq->subtype = nqs->subtype;
	nq->time = nqs->time;
	nq->jid = nqs->jid;
	nq->nxt = NULL;

	return;
}

/*
 *	nqs_hdr() - Print the NQS report header.
 */
void
nqs_hdr()
{
	printf("\n\nNQS Information Only\n");
	printf("====================\n");
	printf("  Request Name          Job Id          Status       Time\n");
	printf("================  ==================   =========    ");
	printf("=================\n");

	return;
}

/*
 *	rpt_nqs() - Print out NQS informational records
 */
void
rpt_nqs()
{
	char	status[9];
	struct	tm	*ttime;
	struct	nqsja	*nqptr;
	char	time_buf[TIMEBUF_MAX];

	if (nqhdr->head != NULL) {
		nqs_hdr();
		for (nqptr = nqhdr->head; nqptr != NULL; nqptr = nqptr->nxt) {
			switch(nqptr->subtype) {

			case (NQ_INIT_START):
				strcpy(status,"INITIATED");
				break;

			case (NQ_INIT_RESTART):
				strcpy(status,"RESTARTED");
				break;

			case (NQ_INIT_RERUN):
				strcpy(status,"RERUN");
				break;
			}

			strftime(time_buf, TIMEBUF_MAX, TIME_FORMAT2,
				localtime(&nqptr->time));
			printf("%16s  %#18llx   %9s    %s\n",
				nqptr->reqname, nqptr->jid, status,
				time_buf);
		}
	}

	return;
}
