/*
 * Copyright (c) 2000-2002 Silicon Graphics, Inc and LANL  All Rights Reserved.
 * Copyright (c) 2004 Silicon Graphics, Inc All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it would be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 *
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 * Contact information:  Silicon Graphics, Inc., 1500 Crittenden Lane,
 * Mountain View, CA  94043, or:
 * 
 * http://www.sgi.com 
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

#include <sys/types.h>
#include <sys/param.h>
#include <sys/sysmacros.h>
#include <sys/times.h>

#include <grp.h>
#include <malloc.h>
#include <math.h>
#include <memory.h>
#include <pwd.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#ifdef HAVE_REGCOMP
#ifdef REG_EXTENDED
#define REGCOMP_FLAG    REG_EXTENDED
#else	/* REG_EXTENDED */
#define REGCOMP_FLAG    0
#endif	/* REG_EXTENDED */
#else	/* HAVE_REGCOMP */
#ifdef HAVE_REGCMP
char *regex();
extern char *__loc1;
#endif	/* HAVE_REGCMP */
#endif	/* HAVE_REGCOMP */

#include "acctdef.h"
#include "acctmsg.h"
#include "csa_conf.h"
#include "sbu.h"

#include "csa.h"
#include "csaacct.h"
#include "csaja.h"
#include "extern_ja.h"

#ifndef	MIN
#define	MIN(A,B)	((A) < (B) ? (A) : (B))
#endif

#ifndef	MAX
#define	MAX(A,B)	((A) > (B) ? (A) : (B))
#endif

#define TIMEBUF_MAX	50

int	headerdone = FALSE;	/* header already printed? */

/*	Memory counters */
uint64_t max_corehimem = 0; /* maximum core memory used (Mbytes) */
uint64_t max_cmdchimem = 0; /* maximum CMD core memory used (Mbytes) */
uint64_t max_mppchimem = 0; /* maximum MPP core memory used (Mbytes) */

uint64_t max_virthimem = 0; /* maximum virtual memory used (Mbytes) */
uint64_t max_cmdvhimem = 0; /* maximum CMD virtual memory used (Mbytes) */
uint64_t max_mppvhimem = 0; /* maximum MPP virtual memory used (Mbytes) */

double	t_coremem = 0.0; /* total core memory integral [Mbytes*secs] */
double	t_cmdcmem = 0.0; /* total CMD core memory integral [Mbytes*secs] */
double	t_mppcmem = 0.0; /* total MPP core memory integral [Mbytes*secs] */

double	t_virtmem = 0.0; /* total virtual memory integral [Mbytes*secs] */
double	t_cmdvmem = 0.0; /* total CMD virtual memory integral [Mbytes*secs] */
double	t_mppvmem = 0.0; /* total MPP virtual memory integral [Mbytes*secs] */

int	t_pgswaps = 0;	/* total # of pages swapped */
int	t_cmdpgswaps = 0; /*total # of CMD pages swapped */
int	t_mpppgswaps = 0; /* total # of MPP pages swapped */

int	t_cmdswaps = 0;	/* total # of CMD times swapped */
int	t_mppswaps = 0;	/* total # of MPP times swapped */

int	t_minflt = 0;	/* total # of minor page faults */

int	t_majflt = 0;	/* total # of major page faults */

/*	Time counters */
double	t_utime = 0.0;	/* total user CPU time (seconds) */
double	t_cmdutime = 0.0; /* total CMD user CPU time (secs) */
double	t_mpputime = 0.0; /* total MPP user CPU time (secs) */

double	t_wutime = 0.0;	/* total weighted user CPU time (seconds) */
double	t_cmdwutime = 0.0; /* total weighted user CPU time (seconds) */
double	t_mppwutime = 0.0; /* total weighted user CPU time (seconds) */

double	t_stime = 0.0;	/* total system CPU time [seconds] */
double	t_cmdstime = 0.0; /* total CMD system CPU time [secs] */
double	t_mppstime = 0.0; /* total MPP system CPU time [secs] */

double	t_iobwtime = 0.0; /* total block I/O wait time [seconds] */
double	t_cmdiobwtime = 0.0; /* total CMD block I/O wait time [secs] */
double	t_mppiobwtime = 0.0; /* total MPP block I/O wait time [secs] */

double	t_iorwtime = 0.0; /* total raw I/O wait time [seconds] */
double	t_cmdiorwtime = 0.0; /* total CMD raw I/O wait time [secs] */
double	t_mppiorwtime = 0.0; /* total MPP raw I/O wait time [secs] */

/*	Multi-tasking counters */
int	numcpu;		/* current # of used multi-tasking CPUs */
int	maxcpu;		/* maximum # of available multi-tasking CPUs */
int	max_numcpu = 0;	/* maximum # of used multi-tasking CPUs */

/*	Multi-PE counters */
int	t_mpbesu = 0;	/* total MPP BESU usage */
int	t_mpbesuhw = 0;	/* total MPP BESU high water */
int	t_mpppe = 0;	/* total MPP PE usage */
int	t_mpppehw = 0;	/* total MPP PE high water */
double	t_mppctime = 0.0;  /* total MPP connect CPU time [secs] */
double	t_bmppctime = 0.0; /* base PE MPP connect CPU time [secs] */

/*	I/O counters */
int64_t	t_chr = 0;	/* total # of characters read (Mbytes) */
int64_t	t_cmdchr = 0;	/* total # of CMD characters read (Mbytes) */
int64_t	t_mppchr = 0;	/* total # of MPP characters read (Mbytes) */

int64_t t_chw = 0;	/* total # of characters written (Mbytes) */
int64_t t_cmdchw = 0;	/* total # of CMD characters written (Mbytes) */
int64_t t_mppchw = 0;	/* total # of MPP characters written (Mbytes) */

int64_t t_bkr = 0;	/* total # of blocks read */
int64_t t_cmdbkr = 0;	/* total # of CMD blocks read */
int64_t t_mppbkr = 0;	/* total # of MPP blocks read */

int64_t t_bkw = 0; 	/* total # of blocks written */
int64_t t_cmdbkw = 0;	/* total # of CMD blocks written */
int64_t t_mppbkw = 0;	/* total # of MPP blocks written */

int64_t	t_scr = 0;	/* total # of logical read I/O requests */
int64_t	t_cmdscr = 0;	/* total # of CMD logical read I/O requests */
int64_t	t_mppscr = 0;	/* total # of MPP logical read I/O requests */

int64_t t_scw = 0;	/* total # of logical write I/O requests */
int64_t t_cmdscw = 0;	/* total # of CMD logical write I/O requests */
int64_t t_mppscw = 0;	/* total # of MPP logical write I/O requests */

/*	SBU counters */
double	t_sbu = 0.0;	/* total system billing units */
double	t_cmdsbu = 0.0;	/* total CMD system billing units */
double	t_mppsbu = 0.0;	/* total MPP system billing units */

double	t_mttime[ACCT_MAXCPUS];	/* total user CPU time with i + 1 */
				/* CPUs connected (seconds) */

/*	Command counters */
int	t_Ncmd = 0;	/* total # of CSA commands processed */
int	t_Ncmdmem = 0;	/* total # of CSA Memory records processed */
int	t_Ncmdio = 0;	/* total # of CSA Input/Output records processed */
int	t_Ncmdmt = 0;	/* total # of CSA Multi-tasking records processed */
int	t_Ncmdmpp = 0;	/* total # of CSA multi-PE records processed */
int	t_Ncmdcmd = 0;	/* total # of CMD commands processed */

int	t_NIcmd = 0;	/* total # of Incremental records encountered */
int	t_NIcmdmpp = 0;	/* total # of Incremental multi-PE records processed */
int	t_NIcmdcmd = 0;	/* total # of Incremental CMD commands processed */

time_t	t_begin = 0;	/* beginning time (seconds) of first cmd */
time_t	t_end= 0;	/* ending time (seconds) of last command */

extern	double	*mpp_expand(struct acctmpp *);
extern	double	*mpp_percentage(struct acctent *, double *);
extern	void	mpp_basic_stats(struct acctmpp *);
extern	void	mpp_extended_stats(struct acctmpp *, double *);
extern	void	mpp_dump(struct acctmpp *, double *);

/*
 *	matchnames() - See if we have a match for the current command
 *	with any of the selection names.
 */
int
#ifdef HAVE_REGCOMP
matchnames(char *comm, regex_t **names)
#else	/* HAVE_REGCOMP */
#ifdef HAVE_REGCMP
matchnames(char *comm, char **names)
#endif	/* HAVE_REGCMP */
#endif	/* HAVE_REGCOMP */
{
	char	command[17];
	int	ind;
#ifdef HAVE_REGCOMP
	regmatch_t rm;
#endif

	for(ind = 0; ind < 16 && comm[ind] != ' ' && comm[ind] != '\0'; ind++) {
		command[ind] = comm[ind];
	}
	command[ind] = '\0';

	while (*names) {
#ifdef HAVE_REGCOMP
		if (regexec(*names++, command, 1, &rm, REGCOMP_FLAG)
			!= REG_NOMATCH) {
#else	/* HAVE_REGCOMP */
#ifdef HAVE_REGCMP
		if (regex(*names++, command)) {
#endif	/* HAVE_REGCMP */
#endif	/* HAVE_REGCOMP */
			return(TRUE);
		}
	}

	return(FALSE);
}


/*
 *	printcmdhdr() - Print header for the command report.
 */
void
printcmdhdr(int l_opt)
{
	printf("\n\n");
	printf("Job Accounting - Command Report\n");
	printf("===============================\n\n");

	/* print first line of header */
	printf("%-17.17s", "    Command   ");
	printf(" Started  ");
	printf(" Elapsed "
		"   User CPU"
		"   Sys CPU");
	if (c_opt) {
		printf("    Blk I/O ");
		printf("   Raw I/O   ");
	}
	if (l_opt) {
		printf(" CPU MEM        Characters          Blocks  ");
		printf("        Logical I/O    ");

		if (h_opt) {
			printf(" CoreMem  VirtMem ");
		}
		printf(" Ex");
	}
	printf("\n");

	/* print second line of header */
	printf("%-17.17s", "     Name    ");		/* Command */
	printf("   At    ");			/* Started At */
	printf("  Seconds "				/* Elapsed */
		"   Seconds"				/* User CPU */
		"    Seconds");				/* Sys CPU */
	if (c_opt) {
		printf("   Wait Secs ");	/* Blk I/O Wait */
		printf(" Wait Secs ");		/* Raw I/O Wait */
	}
	if (l_opt) {
		printf(" Avg Mbytes "		/* CPU Mem */
			"  Read     Written "	/* Chars */
			"   Read   Written " 	/* Blks */
			"   Read     Write  ");	/* Log I/O */

		if (h_opt) {
			printf(" HiValue  HiValue "); /* Memory */
		}
		printf(" St"			/* Exit status */
			"  Ni "
			"Fl ");
	}
	if (c_opt) {
		printf(" SBU's\n");
	} else {
		printf("\n");
	}

	/* print last line of header - the separators */
	printf("%-17.17s", "===============");		/* Command Name */
	printf("======== ");			/* Started At */
	printf(" =========="			/* Elapsed Time */
		" =========="			/* User CPU Seconds */
		" ==========");			/* Sys CPU Seconds */
	if (c_opt) {
		printf(" ==========");		/* Blk I/O Wait Seconds */
		printf(" ==========");		/* Raw I/O Wait Seconds */
	}
	if (l_opt) {
		printf(" =========="		/* CPU Mem Avg Mwds */
			" ========= ========="	/* Chars read/written */
			" ======== ========"	/* Blks read/written */
			" ======== ========"); /* Log I/O Requests */

		if (h_opt) {
			printf(" ======== ========"); /* Memory */
		}
		printf(" ==="			/* Exit St */
			" ==="			/* Ni */
			" ==");			/* Fl */
	}
	if (c_opt) {
		printf(" =======\n");		/* SBU's */
	} else {
		printf("\n");
	}

	return;
}

/*
 *	printohdr() - Print header for the other command report.
 */
void
printohdr()
{
	printf("\n\n");
	printf("Job Accounting - Other (Alternate) Command Report\n");
	printf("=================================================\n\n");

	/* print first line of header */
	printf("%-17.17s", "Command");
	printf(" %8.8s", "Started ");
	printf(" %11.11s", "  Elapsed  ");
	printf(" %6.6s", " Proc ");
	printf(" %6.6s", "Parent");
	printf(" %8.8s", " CoreMem ");
	printf(" %8.8s", " VirtMem ");
	printf(" \n");

	/* print second line of header */
	printf("%-17.17s", "  Name");		/* Command */
	printf(" %8.8s", "   At   ");
	printf(" %11.11s", "  Seconds  ");
	printf(" %6.6s", "  ID  ");
	printf(" %6.6s", "ProcID");
	printf(" %8.8s", " HiWater");
	printf(" %8.8s", " HiWater");
	printf(" \n");

	/* print last line of header -- the separators */
	printf("%-17.17s", "===============");		/* Command */
	printf(" %8.8s", "========");
	printf(" %11.11s", "===========");
	printf(" %6.6s", "======");
	printf(" %6.6s", "======");
	printf(" %-8.8s", "========");
	printf(" %-8.8s", "========");
	printf(" \n");

	return;
}



/*
 *	pr_process() - Process a pacct record and accumulate totals.
 *	Returns TRUE if no error in pacct record, otherwise return FALSE.
 */
int
pr_process()
{
	int		ind;

	time_t	btime;			/* process beginning time [seconds] */
	double	etime;			/* elapsed time [seconds] */
	uint64_t  elaptime;		/* elapsed time [usecs] */
	uint64_t  systime;		/* system time [usecs] */
	double	stime;			/* system CPU time [seconds] */
	uint64_t  usertime;		/* user time [usecs] */
	double	utime;			/* user CPU time [seconds] */
	double	iobwtime = 0.0;		/* Block I/O wait time [seconds] */
	double	iorwtime = 0.0;		/* Raw I/O wait time [seconds] */

	double	mem = 0.0;		/* memory integral [Mbyte-sec] */
	double	coremem = 0.0;		/* core memory integral [Mbyte-sec] */
	double	virtmem = 0.0;		/* virt memory integral [Mbyte-sec] */
	uint64_t corehimem = 0;		/* core memory hiVal [Mbytes] */
	uint64_t virthimem = 0;		/* virtual memory hiVal [Mbytes] */

	u_int64_t  mtime[ACCT_MAXCPUS];	/* temporary array of ac_mttimes */
	double	mttime[ACCT_MAXCPUS];	/* user CPU time with i + 1 CPUs */
					/* connected (seconds) */

	int64_t chr = 0;		/* # of characters read */
	int64_t	chw = 0;		/* # of characters written */
	int64_t bkr = 0;		/* # of blocks read */
	int64_t bkw = 0;		/* # of blocks written */
	int64_t	scr = 0;		/* # of logical I/O read requests */
	int64_t scw = 0;		/* # of logical I/O write requests */
	double	*exp_mpp;		/* expanded mpp data array */
	int	pgswaps = 0;		/* # of pages swapped */
	int	minflt = 0;		/* # of minor page faults */
	int	majflt = 0;		/* # of major page faults */

	double	sbu = 0.0;		/* system billing units */
	char	time_buf[TIMEBUF_MAX];	/* time string buffer */

	static	int	firsttime = 1;	/* first time in this routine? */

	if (firsttime) {
		for (ind = 0; ind < ACCT_MAXCPUS; ind++) {
			t_mttime[ind] = 0.0;
		}

		firsttime = 0;
	}

	/*
	 *  See if the kernel had problems writing to the job accounting file.
	 *  If the record is not now a CSA base record, then what is it?
	 */
	if (!acctrec.csa) {
		int	type = acctrec.prime->ah_type;

		if (db_flag) {
			fprintf(stderr,
			      "# ERROR  (%5o): An unexpected record type was detected as a primary type.\n",
				type);
		}
		acct_err(ACCT_WARN,
		       _("An unexpected record type (%d) was detected as a primary type in the\n   '%s' routine."),
			type, "pr_process()");
		return(TRUE);
	}

	/*
	 * Reject this entry if the user is being selective and does
	 * not want to see this one.
	 */
	if (g_opt && (acctrec.csa->ac_gid != s_gid) ||
	    j_opt && (acctrec.csa->ac_jid != s_jid) ||
	    p_opt && (acctrec.csa->ac_prid!= s_prid) ||
	    u_opt && (acctrec.csa->ac_uid != s_uid) ||
	    a_opt && (acctrec.csa->ac_ash != s_ash) ||
	    n_opt && !matchnames(acctrec.csa->ac_comm, names)) {

		if (db_flag > 3) {
		    fprintf(stderr,
		      "%s: record deselected - uid(%d, %d), gid(%d, %d), prid(%lld, %lld), ash(0x%llx, 0x%llx), jid(0x%llx, 0x%llx).\n",
			Prg_name,
			acctrec.csa->ac_uid, s_uid,
			acctrec.csa->ac_gid, s_gid,
			acctrec.csa->ac_prid, s_prid,
			acctrec.csa->ac_ash, s_ash,
			acctrec.csa->ac_jid, s_jid);
		}
		return(TRUE);
	}

	/*
	 * Convert the stats into something meaningful.
	 */
	elaptime = acctrec.csa->ac_etime;
	etime = TIME_2_SECS(elaptime);
	usertime = acctrec.csa->ac_utime;
	utime    = TIME_2_SECS(usertime);
	if (db_flag > 1) {
		fprintf(stderr, "%s: utime data - USEC(%llu), SEC(%10.2f)\n",
			"pr_process()",usertime, utime);
	}
	systime  = acctrec.csa->ac_stime;
	stime    = TIME_2_SECS(systime);

	if (db_flag > 1) {
		fprintf(stderr, "process ID information.\n");
	}
	if (s_ash == -1) {
		s_ash = acctrec.csa->ac_ash;

	} else if (acctrec.csa->ac_ash != s_ash) {
		s_ash = -2;
	}

	if (s_prid == -1) {
		s_prid = acctrec.csa->ac_prid;

	} else if (acctrec.csa->ac_prid != s_prid) {
		s_prid = -2;
	}

	if (s_gid == -1) {
		s_gid = acctrec.csa->ac_gid;

	} else if (acctrec.csa->ac_gid != s_gid) {
		s_gid = -2;
	}

	if (s_jid == -1) {
		s_jid = acctrec.csa->ac_jid;

	} else if (acctrec.csa->ac_jid != s_jid) {
		s_jid = -2;
	}

	if (s_uid == -1) {
		s_uid = acctrec.csa->ac_uid;

	} else if (acctrec.csa->ac_uid != s_uid) {
		s_uid = -2;
	}

	/*  Process for a Memory record. */
	if (acctrec.mem) {
		t_Ncmdmem++;
		if (db_flag > 1) {
			fprintf(stderr, "process Memory information.\n");
		}
		corehimem  = acctrec.mem->ac_core.himem;
		if (corehimem > max_corehimem) {
			max_corehimem = corehimem;
		}
		virthimem  = acctrec.mem->ac_virt.himem;
		if (virthimem > max_virthimem) {
			max_virthimem = virthimem;
		}
		pgswaps = acctrec.mem->ac_pgswap;
		minflt = acctrec.mem->ac_minflt;
		majflt = acctrec.mem->ac_majflt;

		switch(MEMINT) {

		case 1:
			coremem = TIME_2_SECS(acctrec.mem->ac_core.mem1);
			virtmem = TIME_2_SECS(acctrec.mem->ac_virt.mem1);
			mem  = coremem + virtmem;
			break;

		case 2:
			coremem = TIME_2_SECS(acctrec.mem->ac_core.mem2);
			virtmem = TIME_2_SECS(acctrec.mem->ac_virt.mem2);
			mem  = coremem + virtmem;
			break;

		case 3:
			coremem = TIME_2_SECS(acctrec.mem->ac_core.mem3);
			virtmem = TIME_2_SECS(acctrec.mem->ac_virt.mem3);
			mem  = coremem + virtmem;
			break;

		default:
			acct_err(ACCT_ABORT,
			       _("An invalid value for MEMINT (%d) was found in the configuration file.\n  MEMINT must be between 1 and 3."),
				MEMINT);
		}
	}

	/*  Process for a I/O record. */
	if (acctrec.io) {
		t_Ncmdio++;
		if (db_flag > 1) {
			fprintf(stderr, "process I/O information.\n");
		}
  		iobwtime  = TIME_2_SECS(acctrec.io->ac_bwtime);
  		iorwtime = TIME_2_SECS(acctrec.io->ac_rwtime);

		chr = acctrec.io->ac_chr;
		chw = acctrec.io->ac_chw;
		bkr = acctrec.io->ac_bkr;
		bkw = acctrec.io->ac_bkw;
		scr = acctrec.io->ac_scr;
		scw = acctrec.io->ac_scw;
	}

	/*  Process for a Multi-tasking record. */
	if (acctrec.mt) {
		t_Ncmdmt++;
		get_mttime(mtime, &acctrec);
		numcpu = acctrec.mt->ac_numcpu;
		maxcpu = acctrec.mt->ac_maxcpu;
		if (db_flag > 1) {
		    fprintf(stderr,
		      "process Multi-Tasking information, numcpu(%d), maxcpu(%d).\n",
			numcpu, maxcpu);
		} 
		numcpu = (numcpu < 0 || numcpu > ACCT_MAXCPUS) ? 0 : numcpu;
		max_numcpu = MAX(max_numcpu, numcpu);
		for(ind = 0; ind < numcpu; ind++) {
			mttime[ind] = TIME_2_SECS(mtime[ind]);
		}
	}

	if (db_flag > 1) {
		fprintf(stderr, "Compute the SBUs.\n");
	}
  	sbu = process_sbu(&acctrec);
	if (acctrec.csa->ac_hdr1.ah_flag & AINC) {
		if (db_flag > 1) {
		    fprintf(stderr, "Incremental accounting record found.\n");
		}
		t_NIcmd++;
		if (acctrec.mpp) {
			t_NIcmdmpp++;
		} else {
			t_NIcmdcmd++;
		}

	/* Accumulate totals. */
	} else {
		if (db_flag > 1) {
			fprintf(stderr, "accumulate totals.\n");
		}
		t_Ncmd++;
		t_utime += utime;
		t_stime += stime;

		/* XXX when have multitasking records, uncomment this 
		t_wutime += TIME_2_SECS(ucputime(&acctrec));
		*/
		t_wutime += utime;

  		t_iobwtime += iobwtime;
  		t_iorwtime += iorwtime;

		/* this needs work once MPP records are written */
		if (acctrec.mpp) {
			struct acctmpp	*mpp;

			if (db_flag > 1) {
				fprintf(stderr,
				      "Process a Multi-PE record.\n");
			}
			mpp = acctrec.mpp;
			t_Ncmdmpp++;
			/* time information */
			t_mpputime += utime;
			t_mppstime += stime;

			/* memory information */
			if (corehimem > max_mppchimem) {
				max_mppchimem = corehimem;
			}
			if (virthimem > max_mppvhimem) {
				max_mppvhimem = virthimem;
			}
			t_mppcmem += coremem;
			t_mppvmem += virtmem;

			/* mpp information */
			t_mpbesu   += mpp->ac_mpbesu;
			t_mpbesuhw  = MAX(t_mpbesuhw,mpp->ac_mpbesu);
			t_mpppe    += mpp->ac_mppe;
			t_mpppehw   = MAX(t_mpppehw,mpp->ac_mppe);

			for(ind = 0; ind < mpp->ac_mppe; ind++) {
				t_mppchr += mpp->ac_mpp[ind].io;
			}

			/* I/O information */
			t_mppchr += chr;
			t_mppchw += chw;
			t_mppbkr += bkr;
			t_mppbkw += bkw;
			t_mppscr += scr;
			t_mppscw += scw;

			t_mppsbu += sbu;

		} else {
			t_Ncmdcmd++;
			/* time information */
			t_cmdutime += utime;
			t_cmdstime += stime;

			/* memory information */
			if (corehimem > max_cmdchimem) {
				max_cmdchimem = corehimem;
			}
			if (virthimem > max_cmdvhimem) {
				max_cmdvhimem = virthimem;
			}
			t_cmdcmem += coremem;
			t_cmdvmem += virtmem;

			/* I/O information */
			t_cmdchr += chr;
			t_cmdchw += chw;
			t_cmdbkr += bkr;
			t_cmdbkw += bkw;
			t_cmdscr += scr;
			t_cmdscw += scw;

			t_cmdsbu += sbu;
		}

		t_coremem += coremem;
		t_virtmem += virtmem;

		t_chr += chr;
		t_chw += chw;
		t_bkr += bkr;
		t_bkw += bkw;
		t_scr += scr;
		t_scw += scw;

		t_sbu += sbu;
		t_pgswaps += pgswaps;
		t_minflt += minflt;
		t_majflt += majflt;

		if (acctrec.mt) {
			for(ind = 0; ind < numcpu; ind++) {
				t_mttime[ind] += mttime[ind];
			}
		}
	}

	if (db_flag > 1) {
		fprintf(stderr, "Process time information.\n");
	}
	btime = acctrec.csa->ac_btime;
	if (!t_begin) {
		t_begin = btime;

	} else if (btime < t_begin) {
		t_begin = btime;
	}

	if (btime + etime > t_end) {
		t_end = btime + etime;
	}

	/* Add this process to the command flow tree. */
	if (f_opt) {
		makeflwt(acctrec.csa);
	}

	/* Issue command report. */
	if (c_opt) {
		if (db_flag > 1) {
			fprintf(stderr, "generate command report.\n");
		}
		if (!r_opt && !headerdone) {
			printcmdhdr(l_opt);
			headerdone = TRUE;
		}

		if (acctrec.csa->ac_hdr1.ah_flag & AINC) {
			printf("I* %-14.14s", acctrec.csa->ac_comm);
		} else {
			printf("%-16.16s", acctrec.csa->ac_comm);
		}

		strftime(time_buf, TIMEBUF_MAX, TIME_FORMAT1,
			localtime(&acctrec.csa->ac_btime));
		printf("%s", time_buf);

		printf(" %11.2f", etime);
		printf(" %10.2f", utime);
		printf(" %10.2f", stime);

		printf(" %10.2f", iobwtime);
		printf(" %10.2f", iorwtime);
		if (l_opt) {
			printf(" %10.2f", (utime + stime) ?
				mem / (utime + stime) : 0.0);
			printf(" %9.3f", (double)chr / (1024 * 1024));
			printf(" %9.3f", (double)chw / (1024 * 1024));
			printf(" %8lld", bkr);
			printf(" %8lld", bkw);
			printf(" %8lld", scr);
			printf(" %8lld", scw);
			if (h_opt) {
				printf(" %8lld", corehimem);
				printf(" %8lld", virthimem);
			}

			/*
			 * ac_stat is only 8 bits, so can't split
			 * into separate code and data portions to display
			*/
			printf(" %3d", acctrec.csa->ac_stat);
			printf(" %3d", acctrec.csa->ac_nice);
			printf(" %c", (acctrec.csa->ac_hdr1.ah_flag & AFORK) ?
					'F' : ' ');
			printf("%c", (acctrec.csa->ac_hdr1.ah_flag & ASU) ?
					'S' : ' ');
		}	/* if (l_opt) */

		printf(" %7.2f\n", sbu);

		if (acctrec.mt) {
			for(ind = 0; ind < numcpu; ind++) {
				printf("#%3d CPU\t\t      %11.2f\n",
					ind + 1, mttime[ind] * (ind + 1));
			}
		}

		if (acctrec.mpp) {
			mpp_basic_stats(acctrec.mpp);
			exp_mpp = mpp_expand(acctrec.mpp);

			if (l_opt && (acctrec.mpp->ac_mppe > 1)) {
				mpp_extended_stats(acctrec.mpp, exp_mpp);
			}

			if (exp_mpp != NULL) {
				free(exp_mpp);
			}
		}
	}

	/* issue other command report */
	if (o_opt) {
		if (db_flag > 1) {
	 		fprintf(stderr, "generate OTHER command report.\n");
		}
		if (!r_opt && t_Ncmd == 1) {
			printohdr();
		}

		printf("%-16.16s", acctrec.csa->ac_comm);

		strftime(time_buf, TIMEBUF_MAX, TIME_FORMAT1,
			localtime(&acctrec.csa->ac_btime));
		printf(" %s", time_buf);

		printf(" %11.2f", etime);
		printf(" %6d", acctrec.csa->ac_pid);
		printf(" %6d", acctrec.csa->ac_ppid);

		printf(" %8lld", corehimem);
		printf(" %8lld", virthimem);

		printf(" \n");
	}

	return(TRUE);
}
