/*
 * Copyright (c) 2000-2002 Silicon Graphics, Inc and LANL  All Rights Reserved.
 * Copyright (c) 2004 Silicon Graphics, Inc All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it would be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 *
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 * Contact information:  Silicon Graphics, Inc., 1500 Crittenden Lane,
 * Mountain View, CA  94043, or:
 * 
 * http://www.sgi.com 
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

#include <sys/types.h>
#include <sys/param.h>
#include <sys/sysmacros.h>
#include <sys/times.h>
#include <sys/utsname.h>

#include <grp.h>
#include <malloc.h>
#include <memory.h>
#include <pwd.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#include "acctdef.h"
#include "acctmsg.h"
#include "csa_conf.h"
#include "sbu.h"

#include "csaacct.h"
#include "csaja.h"
#include "extern_ja.h"

#define TIMEBUF_MAX	40

/*
 *	bld_tape() - Add a tape record to the tape list.
 */
void
bld_tape()
{
	struct tapebs	*tape;
	struct tpja	*tb;
	struct tprsv	*rb;
	struct tphdr	*thdr;
	struct rsvhdr	*rhdr;

	if (!d_opt) {
		return;		/* daemon report not specified */
	}

	/*
	 * Reject this entry if the user is being selective and does
	 * not want to see this one.
	 */
	tape = acctrec.tpbs;
	if (j_opt && (tape->jid != s_jid) ||
	    p_opt && (tape->prid!= s_prid) ||
	    u_opt && (tape->uid != s_uid) ) {
		return;
	}

	/*  If this is a base record, put the record in the base list. */
	if (tape->type == TAPE_BASE) {
		thdr = find_tp_base(tape->devname);
		/*  Now add the record to the correct chain. */
		if ((tb = (struct tpja *)malloc(sizeof(struct tpja))) == NULL) {
		    acct_perr(ACCT_ABORT, errno,
		    	_("There was insufficient memory available when allocating '%s'."),
		    	"tpja element");
		}
		if (thdr->last != NULL) {
			thdr->last->nxt = tb;
		}
		thdr->last = tb;
		if (thdr->head == NULL) {
			thdr->head = tb;
		}
		
		tb->uid = tape->uid;
		tb->prid = tape->prid;
		tb->ash = tape->ash;
		tb->jid = tape->jid;
		tb->btime = tape->btime;
		tb->systime = tape->stime;
		tb->usertime = tape->utime;
		(void)strcpy(tb->vsn, tape->vsn);
		tb->nread = tape->nread;
		tb->nwrite = tape->nwrite;
		tb->nxt = NULL;

		return;
	}

	/*
	 *	If this is a reservation record, put it on the
	 *	reservation list.
	 */
	if (tape->type == TAPE_RSV) {
		rhdr = find_tp_rsv(tape->devname);

		/*  Add the reservation record to the correct chain. */
		if ((rb = (struct tprsv *)malloc(sizeof(struct tprsv))) == NULL) {
		    acct_perr(ACCT_ABORT, errno,
		    	_("There was insufficient memory available when allocating '%s'."),
		    	"tprsv element");
		}
		if (rhdr->last != NULL) {
			rhdr->last->nxt = rb;
		}
		rhdr->last = rb;
		if (rhdr->head == NULL) {
			rhdr->head = rb;
		}

		rb->uid = tape->uid;
		rb->prid = tape->prid;
		rb->ash = tape->ash;
		rb->jid = tape->jid;
		rb->btime = tape->btime;
		rb->rtime = tape->rtime;
		rb->nxt   = NULL;

		return;
	}

	/*  Invalid record type - skip it. */
	acct_err(ACCT_WARN,
	       _("An invalid tape record type (%d) was found and ignored."),
		tape->type);

	return;
}


/*
 *	find_tp_base() - Find the correct base chain for the specified
 *	device group.  If none is found, create one.
 *	Return a pointer to the chain.
 */
struct	tphdr *
find_tp_base(char *dname)
{
	struct tphdr	*hdr;
	struct tphdr	*p_hdr;
	int	found;

	if (tp->bs == NULL) {
		if ((hdr = (struct tphdr *)malloc(sizeof(struct tphdr))) ==
			NULL) {
		    acct_perr(ACCT_ABORT, errno,
		    	_("There was insufficient memory available when allocating '%s'."),
		    	"tape base");
		}
		tp->bs = hdr;
		(void)strcpy(hdr->devname, dname);
		hdr->head = hdr->last = NULL;
		hdr->nxt = NULL;

	/*  Look for the correct chain.  If not found, create one. */
	} else {
		hdr = tp->bs;
		found = 0;
		while (hdr != NULL) {
			if (!strcmp(hdr->devname, dname)) {
				found++;
				break;
			} else {
				p_hdr = hdr;
				hdr = hdr->nxt;
			}
		}

		if (!found) {
			if ((hdr = (struct tphdr *)malloc(
				sizeof(struct tphdr))) == NULL) {
			    acct_perr(ACCT_ABORT, errno,
			    	_("There was insufficient memory available when allocating '%s'."),
			    	"tape base header");
			}
			(void)strcpy(hdr->devname, dname);
			hdr->head = hdr->last = NULL;
			hdr->nxt = NULL;
			p_hdr->nxt = hdr;
		}
	}

	return(hdr);
}

/*
 *	find_tp_rsv() - Find the correct reservation chain for the
 *	specified device group.  If none is found, create one.
 *	Return a pointer to the chain.
 */
struct	rsvhdr *
find_tp_rsv(char *dname)
{
	struct rsvhdr	*hdr;
	struct rsvhdr	*p_hdr;
	int		found;

	if (tp->rsv == NULL) {
		if ((hdr = (struct rsvhdr *)malloc(sizeof(struct rsvhdr))) ==
			NULL) {
		    acct_perr(ACCT_ABORT, errno,
		    	_("There was insufficient memory available when allocating '%s'."),
		    	"tape reservation header");
		}
		tp->rsv = hdr;
		(void)strcpy(hdr->devname, dname);
		hdr->head = hdr->last = NULL;
		hdr->nxt = NULL;

	/*  Look for the correct chain.  If not found, create one. */
	} else {
		hdr = tp->rsv;
		found = 0;
		while (hdr != NULL) {
			if (!strcmp(hdr->devname, dname)) {
				found++;
				break;
			} else {
				p_hdr = hdr;
				hdr = hdr->nxt;
			}
		}

		if (!found) {
			if ((hdr = (struct rsvhdr *)malloc(
				sizeof(struct rsvhdr))) == NULL) {
			    acct_perr(ACCT_ABORT, errno,
			    	_("There was insufficient memory available when allocating '%s'."),
			    	"tape base header");
			}
			(void)strcpy(hdr->devname, dname);
			hdr->head = hdr->last = NULL;
			hdr->nxt = NULL;
			p_hdr->nxt = hdr;
		}
	}

	return(hdr);
}

/*
 *	tp_bs_hdr() - Print the header for the tape base records.
 */
void
tp_bs_hdr()
{
	printf(" Device   Started    Kbytes      Kbytes    ");
	printf(" Usertime     Systime       Internal    \n");

	printf(" Group      At        Read      Written    ");
	printf("  Seconds     Seconds          Vsn      \n");

	printf("========  ======== ==========  ==========  ");
	printf("==========  ==========  ================\n");

	return;
}

/*
 *	tp_rsv_hdr() - Print the header for the tape reservation records.
 */
void
tp_rsv_hdr()
{
	printf(" Device   Release   Rtime/Dev \n");
	printf(" Group      Time     Minutes  \n");
	printf("========  ========  ==========\n");

	return;
}

/*
 *	rpt_tape() - Print out the indivdual tape records and/or the 
 *	tape summary.
*/
void
rpt_tape()
{
	struct	rsvhdr	*rhdr;
	struct	tm	*ttime;
	struct	tphdr	*bshdr;
	struct	tpja	*bsptr;
	struct	tpja	bstot;		/* base totals */
	struct	tprsv	*rptr;
	struct	tprsv	rsvtot;		/* reservation totals */
	char	time_buf[TIMEBUF_MAX];

	if (tp->rsv == NULL && tp->bs == NULL) {
		printf("\nNo tape usage\n");
		return;
	}

	if (c_opt || s_opt) {
		printf("\n\nTape Accounting\n");
		printf("===============\n\n");
		if (tp->bs != NULL) {
			tp_bs_hdr();
			for(bshdr = tp->bs; bshdr != NULL; bshdr = bshdr->nxt){
				memset((char *)&bstot,'\0',sizeof(struct tpja));
				for(bsptr = bshdr->head; bsptr != NULL;
						 bsptr = bsptr->nxt) {
					/*
					 *	Output individual records.
					 */
					if (c_opt) {
						time_t	btime;

						btime = bsptr->btime;
						strftime(time_buf, TIMEBUF_MAX,
							TIME_FORMAT1,
							localtime(&btime));
						printf("%-8.8s %s",
							bshdr->devname,
							time_buf);
						printf("%10.3f  ", (double)
							bsptr->nread/1024.0);
						printf("%10.3f  ", (double)
							bsptr->nwrite/1024.0);
						printf("%10.3f  ", TIME_2_SECS(
							bsptr->usertime));
						printf("%10.3f  ", TIME_2_SECS(
							bsptr->systime));
						printf("%16.16s", bsptr->vsn);
						printf("\n");
					}

					/*
					 *	Accumulate totals.
					 */
					if (s_opt) {
						bstot.systime += bsptr->systime;
						bstot.usertime +=
						    bsptr->usertime;
						bstot.nread += bsptr->nread;
						bstot.nwrite += bsptr->nwrite;
					}
				}

				/*  Output the totals for this device group. */
				if (s_opt) {
					if (c_opt) {
						printf("          -------- "
							"----------  "
							"----------  "
							"----------  "
							"----------\n");
					}
					printf("%-8.8s  Totals  ",
						bshdr->devname);
					printf("%10.3f  %10.3f  ",
						(double)bstot.nread/1024.0,
						(double)bstot.nwrite/1024.0);
					printf("%10.3f  %10.3f",
						TIME_2_SECS(bstot.usertime),
						TIME_2_SECS(bstot.systime));
					printf("\n\n");
				}
			}
			printf("\n");
		}

		if (tp->rsv != NULL) {
			tp_rsv_hdr();
			for(rhdr = tp->rsv; rhdr != NULL; rhdr = rhdr->nxt) {
				memset((char *)&rsvtot,'\0',
					sizeof(struct tprsv));
				for(rptr = rhdr->head; rptr != NULL;
						rptr = rptr->nxt) {
					if (c_opt) {
						time_t	btime;

						btime = rptr->btime;
						strftime(time_buf, TIMEBUF_MAX,
							TIME_FORMAT1, 
							localtime(&btime));
						printf("%-8.8s %s",
							rhdr->devname,
							time_buf);
						printf("  %10.3f  ", (double)
						      TIME_2_SECS(rptr->rtime) /
							SEC_PER_MIN);
						printf("\n");
					}

					/*
					 *	Accumulate totals.
					 */
					if (s_opt) {
						rsvtot.rtime += rptr->rtime;
					}
				}
				/*  Output reservation totals. */
				if (s_opt) {
					if (c_opt) {
						printf("          --------  "
							"----------\n");
					}
					printf("%-8.8s  Totals    ",
						rhdr->devname);
					printf("%10.3f\n\n", (double)
						TIME_2_SECS(rsvtot.rtime) /
						SEC_PER_MIN);
				}
			}
		}
	}

	return;
}
