/*
 * Copyright (c) 2000-2002 Silicon Graphics, Inc and LANL  All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it would be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 *
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 * Contact information:  Silicon Graphics, Inc., 1600 Amphitheatre Pkwy, 
 * Mountain View, CA  94043, or:
 * 
 * http://www.sgi.com 
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <malloc.h>
#include <sys/types.h>
#include <sys/param.h>
#include <unistd.h>
#include <errno.h>

#include "acctdef.h"
#include "sbu.h"
#include "acctmsg.h"

#define	FUNC_NAME	"init_pacct_sbu()"	/* library function name */

/*
 *	Initialize the pacct SBU variables.  This includes the prime
 *	and non-prime time weighting factors.
 */
void
init_pacct_sbu()
{
	char		*resp;		/* return value from config() */

	/*
	 *	Get the prime time weighting factors.
	 */
	P_BASIC = init_real(SBU_P_BASIC, 0.0, TRUE);

	P_TIME = init_real(SBU_P_TIME, 0.0, TRUE);
	P_STIME = init_real(SBU_P_STIME, 0.0, TRUE);
	P_UTIME = init_real(SBU_P_UTIME, 0.0, TRUE);
	P_BWTIME = init_real(SBU_P_BWTIME, 0.0, TRUE);
	P_RWTIME = init_real(SBU_P_RWTIME, 0.0, TRUE);

	P_MEM = init_real(SBU_P_MEM, 0.0, TRUE);
	P_XMEM = init_real(SBU_P_XMEM, 0.0, TRUE);
	P_VMEM = init_real(SBU_P_VMEM, 0.0, TRUE);

	P_IO = init_real(SBU_P_IO, 0.0, TRUE);
	P_BIO = init_real(SBU_P_BIO, 0.0, TRUE);
	P_CIO = init_real(SBU_P_CIO, 0.0, TRUE);
	P_LIO = init_real(SBU_P_LIO, 0.0, TRUE);

	/*
	 *	Get the non-prime time weighting factors.
	 */
	NP_BASIC = init_real(SBU_NP_BASIC, 0.0, TRUE);

	NP_TIME = init_real(SBU_NP_TIME, 0.0, TRUE);
	NP_STIME = init_real(SBU_NP_STIME, 0.0, TRUE);
	NP_UTIME = init_real(SBU_NP_UTIME, 0.0, TRUE);
	NP_BWTIME = init_real(SBU_NP_BWTIME, 0.0, TRUE);
	NP_RWTIME = init_real(SBU_NP_RWTIME, 0.0, TRUE);

	NP_MEM = init_real(SBU_NP_MEM, 0.0, TRUE);
	NP_XMEM = init_real(SBU_NP_XMEM, 0.0, TRUE);
	NP_VMEM = init_real(SBU_NP_VMEM, 0.0, TRUE);

	NP_IO = init_real(SBU_NP_IO, 0.0, TRUE);
	NP_BIO = init_real(SBU_NP_BIO, 0.0, TRUE);
	NP_CIO = init_real(SBU_NP_CIO, 0.0, TRUE);
	NP_LIO = init_real(SBU_NP_LIO, 0.0, TRUE);

	/*
	 *	Get the MPP weighting factors.
	 */
	P_MPP = init_real(SBU_P_MPP, 0.0, TRUE);
	P_MPPPE = init_real(SBU_P_MPPPE, 0.0, TRUE);
	P_MPPBESU = init_real(SBU_P_MPPBESU, 0.0, TRUE);
	P_MPPTIME = init_real(SBU_P_MPPTIME, 0.0, TRUE);

	NP_MPP = init_real(SBU_NP_MPP, 0.0, TRUE);
	NP_MPPPE = init_real(SBU_NP_MPPPE, 0.0, TRUE);
	NP_MPPBESU = init_real(SBU_NP_MPPBESU, 0.0, TRUE);
	NP_MPPTIME = init_real(SBU_NP_MPPTIME, 0.0, TRUE);

	/*
	 *	Initialize the multitasking cpu weights.
	 */
	init_mtask();

	return;
}
#undef	FUNC_NAME
