/*
 * Copyright (c) 2000-2002 Silicon Graphics, Inc and LANL  All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it would be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 *
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 * Contact information:  Silicon Graphics, Inc., 1600 Amphitheatre Pkwy, 
 * Mountain View, CA  94043, or:
 * 
 * http://www.sgi.com 
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#include <errno.h>
#include <malloc.h>
#include <string.h>
#include <stdlib.h>

#include "acctdef.h"
#include "csa_conf.h"
#include "sbu.h"
#include "acctmsg.h"

#define FUNC_NAME	"init_tape_sbu()"	/* library function name */

#define TP_NAME_SIZE	16	/* max # chars in tape device group name */

/*
 *	Get the system parameter values from the configuration file
 *	and create run-time variables for tape sbus.
 */
void
init_tape_sbu()
{
	char	*ptr;
	char	*resp;		/* return value from config() */
	char	token[25];
	char	*valptr;
	int	ind;
	int	siz;
 

	/*
	 *	Get the memory for the sbu array.
	 */
	siz = sizeof(struct tp_sbu_type) * TP_MAXDEVGRPS;
	if ((tp_sbu = (struct tp_sbu_type *)malloc(siz)) == NULL) {
		acct_perr(ACCT_ABORT, errno,
			_("%s: There was insufficient memory available when allocating '%s'."),
			FUNC_NAME,
		    "tape SBU array");
	}
	memset((char *)tp_sbu, '\0', siz);

	/*
	 *	Initialize the sbu array using values from the 
	 *	configuration file.
	 */
	for (ind = 0; ind < TP_MAXDEVGRPS; ind++) {
		sprintf(token, "%s%d", SBU_TAPE, ind);
		if ((resp = config(token)) == NULL) {
			acct_err(ACCT_ABORT,
			       _("%s: The '%s' values are missing from configuration file.\n   Please add the %s values to the configuration file."),
				FUNC_NAME,
			    token, token);
		}
		ptr = resp;
		
		/*
		 *	Get the device group name.
		 */
		if ((valptr = strtok(ptr, __SEPARATORS__)) != NULLCHARPTR) {
			strncpy(tp_sbu[ind].tp_devgrp, valptr, TP_NAME_SIZE);

		} else {
			acct_err(ACCT_ABORT,
			       _("%s: The '%s' %s name is missing from configuration file.\n    Please add the '%s' %s name to the configuration file."),
				FUNC_NAME,
			    token, "device group", token, "device group");
		}
		ptr = NULLCHARPTR;

		/*
		 *	Get the mount sbu.
		 */
		if ((valptr = strtok(ptr, __SEPARATORS__)) != NULLCHARPTR) {
			tp_sbu[ind].tp_mount = atof(valptr);

		} else {
			acct_err(ACCT_ABORT,
			       _("%s: The '%s' %s value is missing from configuration file.\n    Please add the '%s' %s value to the configuration file."),
				FUNC_NAME,
			    token, "mount sbu", token, "mount sbu");
		}
		ptr = NULLCHARPTR;

		/*
		 *	Get the reservation time sbu.
		 */
		if ((valptr = strtok(ptr, __SEPARATORS__)) != NULLCHARPTR) {
			tp_sbu[ind].tp_reserv = atof(valptr);

		} else {
			acct_err(ACCT_ABORT,
			       _("%s: The '%s' %s value is missing from configuration file.\n    Please add the '%s' %s value to the configuration file."),
				FUNC_NAME,
			    token, "reservation sbu", token, "reservation sbu");
		}
		ptr = NULLCHARPTR;

		/*
		 *	Get the read sbu.
		 */
		if ((valptr = strtok(ptr, __SEPARATORS__)) != NULLCHARPTR) {
			tp_sbu[ind].tp_read = atof(valptr);

		} else {
			acct_err(ACCT_ABORT,
			       _("%s: The '%s' %s value is missing from configuration file.\n    Please add the '%s' %s value to the configuration file."),
				FUNC_NAME,
			    token, "read sbu", token, "read sbu");
		}
		ptr = NULLCHARPTR;

		/*
		 *	Get the write sbu.
		 */
		if ((valptr = strtok(ptr, __SEPARATORS__)) != NULLCHARPTR) {
			tp_sbu[ind].tp_write = atof(valptr);

		} else {
			acct_err(ACCT_ABORT,
			       _("%s: The '%s' %s value is missing from configuration file.\n    Please add the '%s' %s value to the configuration file."),
				FUNC_NAME,
			    token, "write sbu", token, "write sbu");
		}

	}
}
#undef	FUNC_NAME
