/*
 * Copyright (c) 2000-2002 Silicon Graphics, Inc and LANL  All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it would be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 *
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 * Contact information:  Silicon Graphics, Inc., 1600 Amphitheatre Pkwy, 
 * Mountain View, CA  94043, or:
 * 
 * http://www.sgi.com 
 */

/*
 *	ucputime returns the amount of user cpu time (in usecs) to charge for.
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "acctdef.h"
#include "sbu.h"
#include "acctmsg.h"

#define FUNC_NAME	"ucputime()"	/* library function name */

double
ucputime(struct acctent *acctrec)
{
	int		ind, ncpu;
	double		sum;
	double		weight;
	u_int64_t	mtime[ACCT_MAXCPUS];

	sum = 0.0;
	if (acctrec->csa) {
		if (acctrec->mt) {
			get_mttime(mtime, acctrec);
			ncpu = acctrec->mt->ac_numcpu;
			ncpu = (ncpu < 0 || ncpu > ACCT_MAXCPUS) ? 0 : ncpu;
			weight = 0.0;
			for (ind = 0; ind < ncpu; ind++) {
				weight += mttime_weight[ind].cpu;
				sum += weight * mtime[ind];
			}
		} else {
			sum = acctrec->csa->ac_utime;
		}
	}

	return(sum);
}
